/*****************************************************************************
 *
 * Copyright (C) 2024 Bjarne von Horn <vh at igh dot de>
 *
 * This file is part of the QtPdCom library.
 *
 * The QtPdCom library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * The QtPdCom library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the QtPdCom Library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "Qml_classes.h"

#include "QtPdCom1.h"

#ifdef QTPDCOM_HAS_LOGIN_MANAGER
#include <QtPdCom1/LoginManager.h>
#endif

QtPdCom::SaslInitializer::SaslInitializer(QObject *parent):
    QObject(parent)
{
#ifdef QTPDCOM_HAS_LOGIN_MANAGER
    QtPdCom::LoginManager::InitLibrary();
#endif
}

QtPdCom::SaslInitializer::~SaslInitializer()
{
#ifdef QTPDCOM_HAS_LOGIN_MANAGER
    QtPdCom::LoginManager::FinalizeLibrary();
#endif
}


using QtPdCom::DefaultProcess;

DefaultProcess *DefaultProcess::s_singletonInstance;
QJSEngine *DefaultProcess::s_engine;

DefaultProcess::DefaultProcess(QObject *parent):
    QtPdCom::Process::Process(parent)
{}

void DefaultProcess::createDefaultInstance(QObject *parent)
{
    s_singletonInstance = new DefaultProcess(parent);
    QtPdCom::Process::setDefaultProcess(DefaultProcess::s_singletonInstance);
}

DefaultProcess *DefaultProcess::create(QQmlEngine *, QJSEngine *engine)
{
    // The instance has to exist before it is used. We cannot replace it.
    Q_ASSERT(s_singletonInstance);

    // The engine has to have the same thread affinity as the singleton.
    Q_ASSERT(engine->thread() == s_singletonInstance->thread());

    // There can only be one engine accessing the singleton.
    if (s_engine) {
        Q_ASSERT(engine == s_engine);
    }
    else {
        s_engine = engine;
    }

    // Explicitly specify C++ ownership so that the engine doesn't delete
    // the instance.
    QJSEngine::setObjectOwnership(
            s_singletonInstance,
            QJSEngine::CppOwnership);
    return s_singletonInstance;
}
