/*****************************************************************************
 *
 * Copyright (C) 2024 Bjarne von Horn <vh at igh dot de>
 *
 * This file is part of the QtPdCom library.
 *
 * The QtPdCom library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * The QtPdCom library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the QtPdCom Library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#pragma once

#include <QQmlEngine>
#include <QtPdCom1/ClientStatisticsModel.h>
#include <QtPdCom1/Process.h>
#include <QtPdCom1/LoginManager.h>
#include <QtPdCom1/MessageModel.h>
#include <QTranslator>

namespace QtPdCom {

struct PdComProcess
{
        Q_GADGET
        QML_FOREIGN(QtPdCom::Process)
        QML_NAMED_ELEMENT(Process)
        QML_ADDED_IN_VERSION(1, 4)
};

struct PdComMessageModel
{
        Q_GADGET
        QML_FOREIGN(QtPdCom::MessageModel)
        QML_NAMED_ELEMENT(MessageModel)
        QML_ADDED_IN_VERSION(1, 4)
};

struct PdClientStatisticsModel
{
        Q_GADGET
        QML_FOREIGN(QtPdCom::ClientStatisticsModel)
        QML_NAMED_ELEMENT(ClientStatisticsModel)
        QML_ADDED_IN_VERSION(1, 4)
};

struct PdComLoginManager
{
        Q_GADGET
        QML_FOREIGN(QtPdCom::LoginManager)
        QML_NAMED_ELEMENT(LoginManager)
        QML_ADDED_IN_VERSION(1, 4)
};

class Q_DECL_EXPORT SaslInitializer: public QObject
{
        Q_OBJECT
        QML_ELEMENT
        QML_ADDED_IN_VERSION(1, 4)

    public:
        explicit SaslInitializer(QObject *parent = nullptr);
        ~SaslInitializer();
};

class Q_DECL_EXPORT DefaultProcess: public QtPdCom::Process
{
        Q_OBJECT
        QML_SINGLETON
        QML_NAMED_ELEMENT(DefaultProcess)
        QML_ADDED_IN_VERSION(1, 4)

    public:
        static DefaultProcess *s_singletonInstance;

        static DefaultProcess *create(QQmlEngine *, QJSEngine *engine);

        static void createDefaultInstance(QObject *parent = nullptr);

    private:
        static QJSEngine *s_engine;
        explicit DefaultProcess(QObject *parent);
};
}  // namespace QtPdCom
